require SCORE_DIR + '/lib/shell_escape'

def parse_meteor(line, expression)
  matched = line.match(expression)
  throw "Meteor line #{line.inspect} does not match #{expression}." unless matched
  matched[1].to_f
end

class METEOR
  def summary_score(request)
    output_file = request.output.perm("meteor_out")
    command = ["java", "-Xmx2G", "-jar", SCORE_DIR + "/meteor-1.4/meteor-1.4.jar", request.hyp.file_name, request.ref.laced_name, "-r", request.ref.number.to_s, "-l", request.language, "-t", request.task]
    command << "-norm" unless request.no_norm
    command << "-lower" if request.no_norm and not request.cased
    system_with_redirect(command, nil, output_file)
    score_lines = File.new(output_file).readlines
    throw "Meteor output should be at least 7 lines" unless score_lines.size > 7
    [parse_meteor(score_lines[-1], /^Final score:\s*([0-9]*\.[0-9]*)$/), parse_meteor(score_lines[-7], /^Precision:\s*(-?[0-9]*\.[0-9]*)$/), parse_meteor(score_lines[-6], /^Recall:\s*(-?[0-9]*\.[0-9]*)$/)].map do |n|
      n.to_f * 100.0
    end
  end
  def summary_format(request)
   ["%5.2f", "%5.2f", "%5.2f"]
  end
  def individual_score(request)
    summary_name = request.output.name("meteor_out")
    summary = File.new(summary_name)
    ["METEOR", "Language", "Format", "Task", "Modules", "Weights", "Parameters"].each do |head|
      line = summary.gets
      throw "Expected line beginning with #{head}; got #{line} in #{summary_name}" unless line[0..(head.size - 1)] == head
    end
    ret = []
    request.hyp.lines.size.times do |i|
      line = summary.gets
      throw "Missing score line in #{summary_name}" unless line
      splt = line.split(/\t/u)
      throw "Too many tabs in line \"#{line}\" of #{summary_name}" unless splt.size == 2
      expected = "Segment #{i+1} score:"
      throw "Expected #{expected}, got #{splt[0]} in #{summary_name}" unless expected == splt[0]
      ret << splt[1].to_f
    end
    [ret]
  end
  def individual_format(request)
    ["%5.2f"]
  end
end
