#Makes NIST XML files from scratch.
require 'cgi'
require SCORE_DIR + '/lib/shell_escape'

#Split an interleaved array into an array indexed by reference then segment
def separate_interleaved_text(from, refs)
  count = (refs || 1)
  out = []
  count.times { out << [] }
  entry = 0
  from.each do |l|
    l.strip!
    out[entry] << l
    entry = 0 if (entry += 1) == count
  end
  out
end

def write_segs(text, out)
  out.puts '<doc docid="bogus" genre="bo">'
  segid = 0
  text.each do |line|
    out.puts '<seg id="' + (segid += 1).to_s + '">' + CGI::escapeHTML(line.strip) + '</seg>'
  end
  out.puts '</doc>'
end

def nistify_faux_source(text, xml)
  xml.puts '<srcset setid="bogus" srclang="any">'
  write_segs(text, xml)
  xml.puts '</srcset>'
end

def nistify_faux_test(text, xml, trglang)
  xml.puts '<tstset setid="bogus" srclang="any" trglang="' + trglang + '" sysid="bogus">'
  write_segs(text, xml)
  xml.puts '</tstset>'
end

def nistify_faux_reference(ref, xml, trglang)
  separate_interleaved_text(ref.laced_lines, ref.number).each_with_index do |r,i|
    xml.puts '<refset setid="bogus" srclang="any" trglang="' + trglang + '" refid="bogus' + i.to_s + '">'
    write_segs(r, xml)
    xml.puts '</refset>'
  end
end

def handle_nist_file(request, suffix)
  name = request.output.temp(suffix)
  file = File.new(name, "w")
  file.puts '<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE mteval SYSTEM "ftp://jaguar.ncsl.nist.gov/mt/resources/mteval-xml-v1.3.dtd">
<mteval>'
  yield file
  file.puts '</mteval>'
  file.close
  name
end

class NIST
  def summary_score(request)
    source_name = handle_nist_file(request, "faux_source.xml") do |file|
      nistify_faux_source(request.hyp.lines, file)
    end
    test_name = handle_nist_file(request, "test.xml") do |file|
      nistify_faux_test(request.hyp.lines, file, request.language)
    end
    ref_name = handle_nist_file(request, "ref.xml") do |file|
      nistify_faux_reference(request.ref, file, request.language)
    end

    bleu_out = request.output.perm("bleu_out")
    args = [SCORE_DIR + "/mteval-v13m.pl", "-r", ref_name, "-s", source_name, "-t", test_name]
    args << "--no-norm" if request.no_norm
    args << "-c" if request.cased or request.no_norm
    system_with_redirect(args, nil, bleu_out)
    score_line = File.new(bleu_out).readlines[7]
    throw "Missing score line in #{bleu_out}" unless score_line
    matched = score_line.match(/^NIST score = *([0-9]*.[0-9]*) *BLEU score = *([0-9]*.[0-9]*) for system "bogus"$/)
    throw "Bad score line in #{bleu_out}: #{score_line}" unless matched
    #IBM, NIST
    [matched[2].to_f * 100.0, matched[1].to_f]
  end
  def summary_format(request)
    ["%5.2f", "%5.2f"]
  end
  def individual_score(request)
    []
  end
  def individual_format(request)
    []
  end
end
